==> drbl-1.8.1-45.tar.bz2: drbl-1.8.1-45/sbin/drbl-live.sh <==
#!/bin/bash
# Author: Steven Shiau <steven _at_ nchc org tw>
# License: GPL

. /opt/drbl/sbin/drbl-conf-functions

# Settings
# try IP from 192.168."100".254
private_IP_alias_eth_def="100"

# functions
USAGE() {
    echo "To start, restart or stop DRBL service in DRBL Live environment."
    echo "Usage:"
    echo "$0 [OPTION] {start|stop|restart}"
    echo "OPTION:"
    echo "-p, --prepare-ocsroot         Prepare clonezilla image home dir also."
    echo "-s, --skip-pause-in-the-end   Skip showing the messsage and pause in the end of execuation."
    echo "start   Start DRBL services now"
    echo "stop    Stop DRBL services now"
    echo "restart Restart DRBL services now"
    echo "Example: To start all DRBL services in this DRBL Live server"
    echo "$0 start"
}

# set locale for clients
set_locale_for_clients() {
  # Some of the code is referred to pakcage casper /usr/share/initramfs-tools/scripts/casper-bottom/14locales
  # We use the locale from server as the default one in client.
  if [ -e /etc/default/locale ]; then
      grep_file=/etc/default/locale
  elif [ -e /etc/environment ]; then # Old locales policy
      grep_file=/etc/environment
  fi
  if [ -n "${grep_file}" ]; then
      # use server's configured locale
      locale=$(grep -s 'LANG=' ${grep_file} | sed s/'LANG='// | tr -d '"' )
  fi
  if [ -z "$locale" ]; then
      # Set a default one
      locale=en_US.UTF-8
  fi
  LANG="${locale}"

  # write the setting in clients.
  for ih in $drblroot/*; do
    echo "LANG=${LANG}" > $ih/${grep_file}
  done
  echo "Since some config files are modified in template client, recreating template tarball for DRBL SSI..."
  gen_ssi_files
} # end of set_locale_for_clients
#
config_network(){
  echo "Detecting the network status..."
  eth_port_no="$(get-nic-devs | wc -l)"
  configured_ip_no="$(get-all-nic-ip -i | wc -w)"
  while [ "$configured_ip_no" -lt "$eth_port_no" ]; do
    [ -n "$(route -n | grep -E "^0.0.0.0")" ] && config_gw_opt="--ignore-gw"
    [ -n "$(grep -E "^[[:space:]]*nameserver[[:space:]]+\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b" /etc/resolv.conf 2>/dev/null)" ] && config_dns_opt="--ignore-dns"
    # find the configured ports, we will exclude them.
    eth_ports="$(get-nic-devs)"
    configured_port_list=""
    for i in $eth_ports; do
      if [ -n "$(get_ip $i)" ]; then
       configured_port_list="$configured_port_list $i"
      fi
    done
    if [ -n "$configured_port_list" ]; then
      ocs-live-netcfg --ip-add " " $config_gw_opt $config_dns_opt --ignore-dv "$configured_port_list"
    else
      ocs-live-netcfg --ip-add " " $config_gw_opt $config_dns_opt
    fi
    configured_ip_no="$(get-all-nic-ip -i | wc -w)"
    [ "$configured_ip_no" -eq "$eth_port_no" ] && break
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo $msg_do_not_assign_default_gw_for_DRBL_NIC
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo "$msg_continue_to_conf_another_NIC"
    echo -n "[Y/n] "
    read continue_conf_eth
    case "$continue_conf_eth" in
       n|N|[nN][oO]) break ;;
    esac
  done

  IP_no="$(get-all-nic-ip -i | wc -w)"
  if [ "$IP_no" -eq 0 ]; then
    [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
    echo $msg_no_ip_address_is_configured
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo "$msg_program_stop"
    exit 1
  elif [ "$IP_no" -eq 1 ]; then
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo "$msg_one_NIC_one_IP_limitation_prompt"
    echo "$msg_if_lease_IP_add_to_pxe_etherboot_only"
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo -n "[Y/n] "
    read limit_pxeclient_ans
    case "$limit_pxeclient_ans" in
       n|N|[nN][oO]) limit_pxeclient="no" ;;
       *) limit_pxeclient="yes" ;;
    esac
    echo -n "$msg_only_one_IP_so_create_alias_IP "
    alias_done="no"
    i="$private_IP_alias_eth_def"
    while [ "$alias_done" = "no" ]; do
      if ! ping -c 3 192.168.$i.254 &>/dev/null; then
        ifconfig eth0:1 192.168.$i.254 netmask 255.255.255.0
        alias_done="yes"
      else
        i=$((i+1))
      fi
    done
    if ! egrep -e "^auto[[:space:]]+eth0:1" /etc/network/interfaces; then
      cat <<-NET_END >> /etc/network/interfaces
auto eth0:1
iface eth0:1 inet static
	address 192.168.$i.254
	netmask 255.255.255.0
NET_END
    fi
    echo "$msg_done!"
  else
    # More than one IP address, so it's not necessary to use alias IP address.
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo $msg_more_than_1_IP_some_for_DRBL_clients
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  fi
} # end of config_network


#################
##### MAIN ######
#################
check_if_root

#
ask_and_load_lang_set

# Default settings.
prepare_ocsroot="no"
pause_in_the_end="yes"
#
while [ $# -gt 0 ]; do
  case "$1" in
    -p|--prepare-ocsroot)
            prepare_ocsroot="yes"
            shift ;;
    -s|--skip-pause-in-the-end)
            pause_in_the_end="no"
            shift ;;
    -*)     echo "${0}: ${1}: invalid option" >&2
            USAGE >& 2
            exit 2 ;;
    *)      break ;;
  esac
done

switch=$1
#
if [ -z "$switch" ]; then
  USAGE
  exit 1
fi

#
if ! is_boot_from_live; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "This command $0 is used in DRBL Live environment only!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  echo "$msg_program_stop"
  exit 1
fi

case "$switch" in
   "start"|"restart")
     [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
     echo "$msg_this_is_for_drbl_server_not_for_clonezilla_server_edition"
     [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
     echo "$msg_are_u_sure_u_want_to_continue"
     echo -n "[Y/n] "
     read drbl_live_ans
     case "$drbl_live_ans" in
        n|N|[nN][oO]) echo "$msg_program_stop" 
	              exit 1 ;;
        *) echo "$msg_ok_let_do_it" ;;
     esac

     # config network first:
     config_network

     #
     if  [ "$prepare_ocsroot" = "yes" ]; then
        echo "Preparing clonezilla image home dir..."
	prep-ocsroot
     fi

     #
     echo "$msg_delimiter_star_line"
     echo $msg_now_continue_to_run_drblpush
     echo $msg_this_might_take_several_minutes
     echo -n $msg_press_enter_to_continue
     read

     # load $client_no_per_NIC and $autologin_passwd_opt for later use
     . $drbl_setup_path/files/ocs/live-hook/ocs-live.conf

     # run drblpush again so that the network interface can be updated. Maybe there is new ethx.
     yes "" | drblpush -i -r 1 -z 1 --not-add-start-drbl-srvi -p $client_no_per_NIC

     echo "$msg_delimiter_star_line"
     if [ "$limit_pxeclient" = "yes" ]; then
       # modify dhcpd.conf, provide IP address to PXE/Etherboot/DRBL client only
       echo "Modifying dhcpd.conf, now only PXE/Etherboot/DRBL client can lease IP address from this server..."
       perl -pi -e 's/^([[:space:]]*)#[[:space:]]*(allow members of "DRBL-Client";)/$1$2/g' /etc/dhcp3/dhcpd.conf
     else
       echo "Modifying dhcpd.conf, now ANY client, not only PXE/Etherboot/DRBL client, can lease IP address from this server..."
       perl -pi -e 's/^([[:space:]]*)#*[[:space:]]*(allow members of "DRBL-Client";)/$1#$2/g' /etc/dhcp3/dhcpd.conf
     fi

     echo "$msg_delimiter_star_line"
     echo "Enable auto login for drbl clients..."
     # enable autologin
     case "$autologin_passwd_opt" in
       "random")
         make_random_password $password_opt
         passwd_autologin="$random_password"
         ;;
       *)
         passwd_autologin="$passwd_autologin_def"
         ;;
     esac
     drbl-login-switch -a --no_nis_update -p "${passwd_autologin}"

     echo "$msg_delimiter_star_line"
     # turn on tcpwrapper
     echo "Setting /etc/hosts.allow..."
     mv -f /etc/hosts.allow /etc/hosts.allow.orig
     client_IP_list="$(get-client-ip-list)"
     client_IP_list="$(echo $client_IP_list)"   # convert to 1 line
     client_IP_list_ar=( $client_IP_list )  # put it in an array
     # We can not list too long in hosts.allow, so put 20 elements in a line.
     nloop="$(echo "scale=0; ${#client_IP_list_ar[@]}/20" | bc -l )"
     echo "# Created by drbl-live.sh" > /etc/hosts.allow
     for i in `seq 1 $nloop`; do
       istart="$(echo "scale=0; 20*$((i-1))" | bc -l)"
       iend="$(echo "scale=0; 20*$((i-1))+19" | bc -l)"
       echo -n "ALL: " >> /etc/hosts.allow
       for j in `seq $istart $iend`; do
         echo -n "${client_IP_list_ar[j]} " >> /etc/hosts.allow
       done
       echo >> /etc/hosts.allow
     done

     # set locales for clients.
     set_locale_for_clients

     #
     echo "Start all DRBL related services by running 'drbl-all-service start'..."
     drbl-all-service start

     #
     echo "$msg_delimiter_star_line"
     echo "$msg_done!"

     if  [ "$pause_in_the_end" = "yes" ]; then
       [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
       echo "$msg_all_set_you_can_turn_on_clients"
       [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
       echo -n $msg_press_enter_to_continue
       # This is a weird problem about tftpd-hpa when it is restared in 'drbl-all-service start' in the above. It's:
       # -------
       # /etc/init.d/tftpd-hpa restart
       # echo -n "press key... "
       # read
       # -------
       # and we always get such an error when running it:
       # -------
       # Restarting HPA's tftpd: in.tftpd.
       # press key... ./td: line 4: read: read error: 0: Resource temporarily unavailable
       # -------
       # Therefore we use < /dev/stdin to avoid tihs.
       read dummy_var < /dev/stdin
     fi
     ;;
   "stop")
     # block all the clients.
     mv -f /etc/hosts.allow /etc/hosts.allow.orig2
     echo "# Disabled by drbl-live.sh" > /etc/hosts.allow

     # stop and remove all services
     echo "Stop DRBL service.."
     drbl-all-service stop
     echo "Removing DRBL-related services..."
     drbl-all-service del
     #
     echo "$msg_delimiter_star_line"
     echo "$msg_done!"
     echo -n $msg_press_enter_to_continue
     read
     ;;
esac

==> drbl-1.8.1-47.tar.bz2: drbl-1.8.1-47/sbin/drbl-live.sh <==
#!/bin/bash
# Author: Steven Shiau <steven _at_ nchc org tw>
# License: GPL

. /opt/drbl/sbin/drbl-conf-functions

# Settings
# try IP from 192.168."100".254
private_IP_alias_eth_def="100"

# functions
USAGE() {
    echo "To start, restart or stop DRBL service in DRBL Live environment."
    echo "Usage:"
    echo "$0 [OPTION] {start|stop|restart}"
    echo "OPTION:"
    echo "-p, --prepare-ocsroot         Prepare clonezilla image home dir also."
    echo "-s, --skip-pause-in-the-end   Skip showing the messsage and pause in the end of execuation."
    echo "-n, --no-prompt-drbl-live     Assume it's running for drbl live. (This is normally only used for clonezilla server edition script ocs-srv-live.sh in DRBL live mode)"
    echo "start   Start DRBL services now"
    echo "stop    Stop DRBL services now"
    echo "restart Restart DRBL services now"
    echo "Example: To start all DRBL services in this DRBL Live server"
    echo "$0 start"
}

# set locale for clients
set_locale_for_clients() {
  # Some of the code is referred to pakcage casper /usr/share/initramfs-tools/scripts/casper-bottom/14locales
  # We use the locale from server as the default one in client.
  if [ -e /etc/default/locale ]; then
      grep_file=/etc/default/locale
  elif [ -e /etc/environment ]; then # Old locales policy
      grep_file=/etc/environment
  fi
  if [ -n "${grep_file}" ]; then
      # use server's configured locale
      locale=$(grep -s 'LANG=' ${grep_file} | sed s/'LANG='// | tr -d '"' )
  fi
  if [ -z "$locale" ]; then
      # Set a default one
      locale=en_US.UTF-8
  fi
  LANG="${locale}"

  # write the setting in clients.
  for ih in $drblroot/*; do
    echo "LANG=${LANG}" > $ih/${grep_file}
  done
  echo "Since some config files are modified in template client, recreating template tarball for DRBL SSI..."
  gen_ssi_files
} # end of set_locale_for_clients
#
config_network(){
  echo "Detecting the network status..."
  eth_port_no="$(get-nic-devs | wc -l)"
  configured_ip_no="$(get-all-nic-ip -i | wc -w)"
  while [ "$configured_ip_no" -lt "$eth_port_no" ]; do
    [ -n "$(route -n | grep -E "^0.0.0.0")" ] && config_gw_opt="--ignore-gw"
    [ -n "$(grep -E "^[[:space:]]*nameserver[[:space:]]+\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b" /etc/resolv.conf 2>/dev/null)" ] && config_dns_opt="--ignore-dns"
    # find the configured ports, we will exclude them.
    eth_ports="$(get-nic-devs)"
    configured_port_list=""
    for i in $eth_ports; do
      if [ -n "$(get_ip $i)" ]; then
       configured_port_list="$configured_port_list $i"
      fi
    done
    if [ -n "$configured_port_list" ]; then
      ocs-live-netcfg --ip-add " " $config_gw_opt $config_dns_opt --ignore-dv "$configured_port_list"
    else
      ocs-live-netcfg --ip-add " " $config_gw_opt $config_dns_opt
    fi
    configured_ip_no="$(get-all-nic-ip -i | wc -w)"
    [ "$configured_ip_no" -eq "$eth_port_no" ] && break
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo $msg_do_not_assign_default_gw_for_DRBL_NIC
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo "$msg_continue_to_conf_another_NIC"
    echo -n "[Y/n] "
    read continue_conf_eth
    case "$continue_conf_eth" in
       n|N|[nN][oO]) break ;;
    esac
  done

  IP_no="$(get-all-nic-ip -i | wc -w)"
  if [ "$IP_no" -eq 0 ]; then
    [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
    echo $msg_no_ip_address_is_configured
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo "$msg_program_stop"
    exit 1
  elif [ "$IP_no" -eq 1 ]; then
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo "$msg_one_NIC_one_IP_limitation_prompt"
    echo "$msg_if_lease_IP_add_to_pxe_etherboot_only"
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
    echo -n "[Y/n] "
    read limit_pxeclient_ans
    case "$limit_pxeclient_ans" in
       n|N|[nN][oO]) limit_pxeclient="no" ;;
       *) limit_pxeclient="yes" ;;
    esac
    echo -n "$msg_only_one_IP_so_create_alias_IP "
    alias_done="no"
    i="$private_IP_alias_eth_def"
    while [ "$alias_done" = "no" ]; do
      if ! ping -c 3 192.168.$i.254 &>/dev/null; then
        ifconfig eth0:1 192.168.$i.254 netmask 255.255.255.0
        alias_done="yes"
      else
        i=$((i+1))
      fi
    done
    if ! egrep -e "^auto[[:space:]]+eth0:1" /etc/network/interfaces; then
      cat <<-NET_END >> /etc/network/interfaces
auto eth0:1
iface eth0:1 inet static
	address 192.168.$i.254
	netmask 255.255.255.0
NET_END
    fi
    echo "$msg_done!"
  else
    # More than one IP address, so it's not necessary to use alias IP address.
    [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
    echo $msg_more_than_1_IP_some_for_DRBL_clients
    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  fi
} # end of config_network


#################
##### MAIN ######
#################
check_if_root

#
ask_and_load_lang_set

# Default settings.
prepare_ocsroot="no"
pause_in_the_end="yes"
prompt_drbl_live="yes"
#
while [ $# -gt 0 ]; do
  case "$1" in
    -p|--prepare-ocsroot)
            prepare_ocsroot="yes"
            shift ;;
    -s|--skip-pause-in-the-end)
            pause_in_the_end="no"
            shift ;;
    -n|--no-prompt-drbl-live)
            prompt_drbl_live="no"
            shift ;;
    -*)     echo "${0}: ${1}: invalid option" >&2
            USAGE >& 2
            exit 2 ;;
    *)      break ;;
  esac
done

switch=$1
#
if [ -z "$switch" ]; then
  USAGE
  exit 1
fi

#
if ! is_boot_from_live; then
  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
  echo "This command $0 is used in DRBL Live environment only!"
  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
  echo "$msg_program_stop"
  exit 1
fi

case "$switch" in
   "start"|"restart")
     if [ "$prompt_drbl_live" = "yes" ]; then
       [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
       echo "$msg_this_is_for_drbl_server_not_for_clonezilla_server_edition"
       [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
       echo "$msg_are_u_sure_u_want_to_continue"
       echo -n "[Y/n] "
       read drbl_live_ans
       case "$drbl_live_ans" in
          n|N|[nN][oO]) echo "$msg_program_stop" 
                        exit 1 ;;
          *) echo "$msg_ok_let_do_it" ;;
       esac
     fi

     # config network first:
     config_network

     #
     if  [ "$prepare_ocsroot" = "yes" ]; then
        echo "Preparing clonezilla image home dir..."
	prep-ocsroot
     fi

     #
     echo "$msg_delimiter_star_line"
     echo $msg_now_continue_to_run_drblpush
     echo $msg_this_might_take_several_minutes
     echo -n $msg_press_enter_to_continue
     read

     # load $client_no_per_NIC and $autologin_passwd_opt for later use
     . $drbl_setup_path/files/ocs/live-hook/ocs-live.conf

     # run drblpush again so that the network interface can be updated. Maybe there is new ethx.
     yes "" | drblpush -i -r 1 -z 1 --not-add-start-drbl-srvi -p $client_no_per_NIC

     echo "$msg_delimiter_star_line"
     if [ "$limit_pxeclient" = "yes" ]; then
       # modify dhcpd.conf, provide IP address to PXE/Etherboot/DRBL client only
       echo "Modifying dhcpd.conf, now only PXE/Etherboot/DRBL client can lease IP address from this server..."
       perl -pi -e 's/^([[:space:]]*)#[[:space:]]*(allow members of "DRBL-Client";)/$1$2/g' /etc/dhcp3/dhcpd.conf
     else
       echo "Modifying dhcpd.conf, now ANY client, not only PXE/Etherboot/DRBL client, can lease IP address from this server..."
       perl -pi -e 's/^([[:space:]]*)#*[[:space:]]*(allow members of "DRBL-Client";)/$1#$2/g' /etc/dhcp3/dhcpd.conf
     fi

     echo "$msg_delimiter_star_line"
     echo "Enable auto login for drbl clients..."
     # enable autologin
     case "$autologin_passwd_opt" in
       "random")
         make_random_password $password_opt
         passwd_autologin="$random_password"
         ;;
       *)
         passwd_autologin="$passwd_autologin_def"
         ;;
     esac
     drbl-login-switch -a --no_nis_update -p "${passwd_autologin}"

     echo "$msg_delimiter_star_line"
     # turn on tcpwrapper
     echo "Setting /etc/hosts.allow..."
     mv -f /etc/hosts.allow /etc/hosts.allow.orig
     client_IP_list="$(get-client-ip-list)"
     client_IP_list="$(echo $client_IP_list)"   # convert to 1 line
     client_IP_list_ar=( $client_IP_list )  # put it in an array
     # We can not list too long in hosts.allow, so put 20 elements in a line.
     nloop="$(echo "scale=0; ${#client_IP_list_ar[@]}/20" | bc -l )"
     echo "# Created by drbl-live.sh" > /etc/hosts.allow
     for i in `seq 1 $nloop`; do
       istart="$(echo "scale=0; 20*$((i-1))" | bc -l)"
       iend="$(echo "scale=0; 20*$((i-1))+19" | bc -l)"
       echo -n "ALL: " >> /etc/hosts.allow
       for j in `seq $istart $iend`; do
         echo -n "${client_IP_list_ar[j]} " >> /etc/hosts.allow
       done
       echo >> /etc/hosts.allow
     done

     # set locales for clients.
     set_locale_for_clients

     #
     echo "Start all DRBL related services by running 'drbl-all-service start'..."
     drbl-all-service start

     #
     echo "$msg_delimiter_star_line"
     echo "$msg_done!"

     if  [ "$pause_in_the_end" = "yes" ]; then
       [ "$BOOTUP" = "color" ] && $SETCOLOR_WARNING
       echo "$msg_all_set_you_can_turn_on_clients"
       [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
       echo -n $msg_press_enter_to_continue
       # This is a weird problem about tftpd-hpa when it is restared in 'drbl-all-service start' in the above. It's:
       # -------
       # /etc/init.d/tftpd-hpa restart
       # echo -n "press key... "
       # read
       # -------
       # and we always get such an error when running it:
       # -------
       # Restarting HPA's tftpd: in.tftpd.
       # press key... ./td: line 4: read: read error: 0: Resource temporarily unavailable
       # -------
       # Therefore we use < /dev/stdin to avoid tihs.
       read dummy_var < /dev/stdin
     fi
     ;;
   "stop")
     # block all the clients.
     mv -f /etc/hosts.allow /etc/hosts.allow.orig2
     echo "# Disabled by drbl-live.sh" > /etc/hosts.allow

     # stop and remove all services
     echo "Stop DRBL service.."
     drbl-all-service stop
     echo "Removing DRBL-related services..."
     drbl-all-service del
     #
     echo "$msg_delimiter_star_line"
     echo "$msg_done!"
     echo -n $msg_press_enter_to_continue
     read
     ;;
esac
